@@ -51,8 +51,6 @@ from inspire_query_parser.utils.visitor_utils import (
     generate_nested_query,
     update_date_value_in_operator_value_pairs_for_fieldname,
     wrap_query_in_nested_if_field_is_nested,
-    is_initial_of_a_name,
-    retokenize_first_names,
     escape_query_string_special_characters
 )
 from inspire_query_parser.visitors.visitor_impl import Visitor
@@ -79,7 +77,7 @@ class ElasticSearchVisitor(Visitor):
     JOURNAL_ART_ID = 'artid'
     JOURNAL_YEAR = 'year'
     JOURNAL_FIELDS_MAPPING = {
-        JOURNAL_TITLE: '.'.join((JOURNAL_FIELDS_PREFIX, JOURNAL_TITLE)),
+        JOURNAL_TITLE: '.'.join((JOURNAL_FIELDS_PREFIX, JOURNAL_TITLE, 'raw')),
         JOURNAL_VOLUME: '.'.join((JOURNAL_FIELDS_PREFIX, JOURNAL_VOLUME)),
         JOURNAL_PAGE_START: '.'.join((JOURNAL_FIELDS_PREFIX, JOURNAL_PAGE_START)),
         JOURNAL_ART_ID: '.'.join((JOURNAL_FIELDS_PREFIX, JOURNAL_ART_ID)),
@@ -859,17 +857,17 @@ class ElasticSearchVisitor(Visitor):
             node.value + \
             ('' if node.value.endswith(ast.GenericValue.WILDCARD_TOKEN) else '*')
 
-        bai_fieldnames = self._generate_fieldnames_if_bai_query(
-            fieldnames,
-            node.value,
-            bai_field_variation=FieldVariations.search,
-            query_bai_field_if_dots_in_name=True
-        )
-
-        query = self._generate_query_string_query(value, fieldnames=bai_fieldnames or fieldnames, analyze_wildcard=True)
-        if self._are_fieldnames_author_or_first_author(bai_fieldnames) or self._are_fieldnames_author_or_first_author(fieldnames):
+        if self._are_fieldnames_author_or_first_author(fieldnames):
+            bai_fieldnames = self._generate_fieldnames_if_bai_query(
+                fieldnames,
+                node.value,
+                bai_field_variation=FieldVariations.search,
+                query_bai_field_if_dots_in_name=True
+            )
+            query = self._generate_query_string_query(value, fieldnames=bai_fieldnames or fieldnames, analyze_wildcard=True)
             return self._generate_nested_author_query(query, fieldnames)
 
+        query = self._generate_query_string_query(value, fieldnames, analyze_wildcard=True)
         return wrap_query_in_nested_if_field_is_nested(query, fieldnames, self.NESTED_FIELDS)
 
     def visit_regex_value(self, node, fieldname="_all"):
